/* **************************************************************************** */
/*                                                                              */
/*  Copyright (c) 2011 University of Southern California (USC)                  */ 
/*                                                                              */
/*  All rights reserved.                                                        */
/*                                                                              */
/*  Permission is given to use, copy, and modify this software for any          */
/*  non-commercial purpose as long as this copyright notice is not removed      */
/*  All other uses, including redistribution in whole or in part, are           */
/*  forbidden without prior written permission.                                 */
/*                                                                              */
/*  This software is provided with absolutely no warranty and no support.       */
/*                                                                              */
/*  For further information contact: pedro@isi.edu                              */
/*                                                                              */
/* **************************************************************************** */

#ifndef STRUCTS_H
#define STRUCTS_H

#include <stdio.h>
#include <stdlib.h>
#include <strings.h>

#include "y.tab.h"

/* --- types of instructions 	--- */
#define T_VOID 		0
#define T_OPER 		1
#define T_UNARY 	2
#define T_COPY 		3
#define T_MSTORE 	4
#define T_MLOAD 	5
#define T_ADDRESSOF 	6
#define T_DEREF 	7
#define T_PTRASSIGN 	8
#define T_COND 		10
#define T_BOOL_COND 	11
#define T_GOTO 		12
#define T_HALT 		13
#define T_PUTPAR 	14
#define T_GETPAR 	15
#define T_CALL 		16
#define T_RETURN 	17
#define T_JUMP 		20
#define T_NOP 		21

/* --- types of Symbols 	--- */
#define NULL_SYM 	00
#define VAR_SYM 	01
#define FUNC_SYM 	02
#define PROC_SYM 	03
#define LAB_SYM 	04
#define REG_SYM 	05

/* --- types of operands 	--- */
#define O_NULL 		00
#define O_STRING 	01
#define O_INTEGER 	02
#define O_DOUBLE 	03
#define O_CHAR 		04

/* --- Codes for Registers  	--- */
/* Positive register are general purpose registers */
#define NUMBER_REGISTERS	32
#define PC_REGISTER		-1
#define SP_REGISTER		-2
#define FP_REGISTER		-3
#define RA_REGISTER		-4

/* --- Special Procedures/Functions --- */
#define EXIT_ADDRESS		-1
#define CREAD_ADDRESS		-2
#define CWRITE_ADDRESS		-3
#define IREAD_ADDRESS		-4
#define IWRITE_ADDRESS		-5
#define FREAD_ADDRESS		-6
#define FWRITE_ADDRESS		-7

/* --- constants 		--- */
#define MAX_MEMORY_SIZE 	1024
#define MAX_STACK_SIZE 		1024
#define MAX_FRAME_STACK_SIZE 	1024
#define MAX_PROG_SIZE 		1024
#define MAX_IDS 		1024
#define MAX_LABELS 		1024
#define MAX_SYMBOL_TAG_SIZE 	64


/* operand/value data structure					*/

typedef struct oper_t {
  int type;		
  union {
    int    ival;
    double dval;
    char*  sval;
    char   cval;
  } u;
} oper_t ; 


/* instruction data structure 					*/

typedef struct inst_t {
  int type; 
  oper_t op1;
  oper_t op2;
  oper_t op3;
  int operator; 
  int line; 
} inst_t;


/* symbol data structure used to capture labels and identifiers		*/

typedef struct symbol_t {
  int type;		/* type of symbol, either VAR_SYMB, PROC_SYM, REG_SYM	*/
  char tag[MAX_SYMBOL_TAG_SIZE]; /* the tag or name of the label or identifier 	*/
  int address;		/* address or PC-relative in the code in the 		*/
			/* case of a label or an address for an id. In  	*/
			/* the current implementation all are global 		*/
  int 	  	size;	/* could be the number of bytes				*/
  int 		attrib;	/* could be some id or auxiliary value like register id	*/
  oper_t* 	initvalue;
} symbol_t;

/* --- Stack Frame Offsets --- */
#define FRAME_POINTER_OFFSET	0
#define PROGRAM_COUNTER_OFFSET	1


/* ----------------------------------------------------------------------- */
extern int 	isNullOperand(oper_t* op);
extern int 	isIntegerOperand(oper_t* op);
extern int 	isDoubleOperand(oper_t* op);
extern int 	isStringOperand(oper_t* op);
extern int 	isCharOperand(oper_t* op);
extern int 	isProgramCounterOperand(oper_t* op);
extern void 	printOperand(oper_t* op);
extern void 	setOperandInteger(oper_t *dst, int v);
extern int 	getIntegerOperandValue(oper_t *src);
extern void 	clearOperand(oper_t* op);
extern void 	copyOperand(oper_t* dst, oper_t* src);
extern oper_t*	createRegisterValOperand(int v);
extern oper_t*	createStringValOperand(char *str);
extern oper_t*	createIntegerValOperand(int v);
extern oper_t*	createDoubleValOperand(double v);
extern oper_t*	createCharValOperand(char v);
extern void 	destroyOperand(oper_t *op);
extern void 	clearInstruction(inst_t *ip);
extern void 	printInstruction(inst_t *ip);
extern symbol_t* createSymbol(int ty, char *s, int addr, int sz, int att);
extern void	initSymbol(symbol_t* sym, int ty, char *s, int addr, int sz, int att);
extern void 	printSymbol(symbol_t *s);
extern void 	setSymbolType(symbol_t *s, int t);
extern void 	setSymbolTag(symbol_t *s, char *t);
extern void 	setSymbolAddress(symbol_t *s, int n);
extern void 	setSymbolSize(symbol_t *s, int v);
extern void 	setSymbolAttribute(symbol_t *s, int att);
extern void 	setSymbolInitValue(symbol_t *s, oper_t* v);

extern int 	isRegisterName(char *n);
extern int 	isRegisterSymbol(symbol_t *st);
extern oper_t*  getRegisterSymbolOperand(char *s);
extern symbol_t* getRegisterSymbol(char *s);

extern int 	isVariableSymbol(symbol_t *st);
extern int 	isFunctionSymbol(symbol_t *st);
extern int 	isFunctionSymbol(symbol_t *st);

/* ----------------------------------------------------------------------- */

#endif
